
// EXR - An ILM OpenEXR plugin to il.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifdef HAVE_CONFIG_H
#include <openlibraries_global_config.hpp>
#endif

#include <openimagelib/plugins/exr/openexr.hpp>
#include <openimagelib/plugins/exr/exr_plugin.hpp>

namespace fs  = boost::filesystem;
namespace il  = olib::openimagelib::il;
namespace opl = olib::openpluginlib;

namespace olib { namespace openimagelib { namespace plugins { namespace EXR {

namespace
{
	typedef il::image<unsigned char, il::surface_format> image_type;
	
	void destroy( image_type* im )
	{ delete im; }

	std::wstring exr_image_type_to_image_type( )
	{
		return L"r16g16b16a16f";
	}

	il::image_type_ptr load_exr( const fs::path& path )
	{
#ifdef HAVE_OPENEXR
		Imf::RgbaInputFile file( path.native_file_string( ).c_str( ) );
		
		Imath::Box2i dw = file.header( ).dataWindow( );
		int width  = dw.max.x - dw.min.x + 1;
		int height = dw.max.y - dw.min.y + 1;
		
		Imf::Array2D<Imf::Rgba> exr_texels;
		exr_texels.resizeErase( height, width );
		
		file.setFrameBuffer( &exr_texels[ 0 ][ 0 ], 1, width );
		file.readPixels( dw.min.y, dw.max.y );

		il::image_type_ptr im = il::allocate( exr_image_type_to_image_type( ), width, height );
		image_type::pointer texels = im->data( );
		
		for( int j = 0; j < height; ++j )
		{
			for( int i = 0; i < width; ++i )
			{
				half h0 = exr_texels[ j ][ i ].r;
				half h1 = exr_texels[ j ][ i ].g;
				half h2 = exr_texels[ j ][ i ].b;
				half h3 = exr_texels[ j ][ i ].a;
				
				( ( unsigned short* ) texels )[ 0 ] = h0.bits( );
				( ( unsigned short* ) texels )[ 1 ] = h1.bits( );
				( ( unsigned short* ) texels )[ 2 ] = h2.bits( );
				( ( unsigned short* ) texels )[ 3 ] = h3.bits( );
					
				texels += 4 * sizeof( unsigned short );
			}

			texels += ( im->pitch( ) - im->linesize( ) ) * sizeof( unsigned short );
		}

		return ilm_exr_tonemap( im, 0.0f, 0.0f, 0.0f, 5.0f );
#else
		return il::image_type_ptr( );
#endif
	}
}

il::image_type_ptr EXR_plugin::load( const fs::path& path )
{ return load_exr( path ); }

bool EXR_plugin::store( const fs::path& /*path*/, const il::image_type_ptr& /*im*/)
{ return false; }

} } } }
