
// fx - An compositing library representation.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

// simple example to test offscreen rendering.

#include <cstdio>
#include <cstdlib>
#include <string>

#ifdef HAVE_CONFIG_H
#include <openlibraries_global_config.hpp>
#endif

#include <openpluginlib/pl/openpluginlib.hpp>
#include <openpluginlib/pl/utf8_utils.hpp>

#include <openeffectslib/fx/render_buffer.hpp>
#include <openeffectslib/fx/openeffectslib_plugin.hpp>

#include <openimagelib/il/openimagelib_plugin.hpp>

#include <GL/glew.h>

#if defined ( WIN32 ) || defined ( HAVE_GL_GLUT_H )
#include <GL/glut.h>
#else
#include <GLUT/glut.h>
#endif

const int WIN_WIDTH	 = 640;
const int WIN_HEIGHT = 480;

const int OFF_WIDTH	 = 512;
const int OFF_HEIGHT = 512;

namespace il  = olib::openimagelib::il;
namespace oel = olib::openeffectslib;
namespace opl = olib::openpluginlib;
namespace fs  = boost::filesystem;

namespace
{
#ifdef WIN32
	const opl::string plugin_path( "./plugins" );
#else
	const opl::string oil_plugin_path( OPENIMAGELIB_PLUGINS );
	const opl::string oel_plugin_path( OPENEFFECTSLIB_PLUGINS );
#endif

	boost::filesystem::path path[ 2 ] = { fs::path( "./test.jpg", fs::native ), fs::path( "./test.png", fs::native ) };

	struct query_traits
	{
		opl::wstring libname( ) const
		{ return L"openeffectslib"; }
		
		opl::wstring to_match( ) const
		{ return L"source"; }

		opl::wstring type( ) const
		{ return L"input"; }
		
		int merit( ) const
		{ return 0; }
	};

	oel::fx::n_operator_ptr op;
}

static bool create_operator( const boost::filesystem::path& path )
{
	typedef opl::discovery<query_traits> oel_discovery;
	
	oel_discovery plugins( ( query_traits( ) ) );
	if( plugins.empty( ) ) return false;
	
	oel_discovery::const_iterator i = plugins.begin( );
	boost::shared_ptr<oel::fx::openeffectslib_plugin> plug = boost::shared_dynamic_cast<oel::fx::openeffectslib_plugin>( i->create_plugin( "" ) );
	if( !plug ) return 0;

	op = plug->create( L"still image source" );
	if( !op ) return false;
	
	opl::v_url* url = op->pointer<opl::v_url>( L"url" );
	url->get( ) = path;

	return true;
}

static void display( void )
{
	glDisable( GL_DEPTH_TEST );
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glEnable( GL_DEPTH_TEST );

	op->update_image( );

	op->bind_as_GL_texture( );

	glViewport( 0, 0, WIN_WIDTH, WIN_HEIGHT );
	
	glBegin( GL_QUADS );
		glMultiTexCoord2f( GL_TEXTURE0, 0.0f, 0.0f );
		glVertex3f( -1.0f, -1.0f, -1.0f );
		glMultiTexCoord2f( GL_TEXTURE0, 1.0f, 0.0f );
		glVertex3f(  1.0f, -1.0f, -1.0f );
		glMultiTexCoord2f( GL_TEXTURE0, 1.0f, 1.0f );
		glVertex3f(  1.0f,  1.0f, -1.0f );
		glMultiTexCoord2f( GL_TEXTURE0, 0.0f, 1.0f );
		glVertex3f( -1.0f,  1.0f, -1.0f );
	glEnd( );	
	
	op->unbind_as_GL_texture( );

#ifndef NDEBUG
	GLenum error;
	while( ( error = glGetError( ) ) != GL_NO_ERROR )
		fprintf( stderr, "GLerror:%s\n", gluErrorString( error ) );
#endif

	glutSwapBuffers( );
}

static void idle( void )
{
	glutPostRedisplay( );
}

static void reshape( int w, int h )
{
	glViewport( 0, 0, w, h );
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	gluPerspective( 45.0f, ( float ) w / h, 0.01f, 1000.0f );
	
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );
	gluLookAt( 0.0f, 0.0f, -3.5f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
}

static void keyboard( unsigned char key, int /*x*/, int /*y*/ )
{
	static int swap = 1;
	
	opl::v_url* url = op->pointer<opl::v_url>( L"url" );
	
	switch( key )
	{
		case 27:
			op.reset( );
			exit( 0 );
			break;

		case 's':
		case 'S':
			url->get( ) = path[ swap ];
			swap = !swap;			
			break;
		
		default:
			break;
	}
}

int main( int argc, char* argv[ ] )
{
	glutInit( &argc, argv );
	glutInitWindowSize( WIN_WIDTH, WIN_HEIGHT );
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA );
	glutCreateWindow( "Framebuffer Object Render To Texture" );
	glutDisplayFunc( display );
	glutIdleFunc( idle );
	glutReshapeFunc( reshape );
	glutKeyboardFunc( keyboard );

	glewInit( );
	
#ifdef WIN32
	opl::init( plugin_path );
#else
	opl::init( oil_plugin_path );
	opl::init( oel_plugin_path );
#endif
	
	if( !create_operator( path[ 0 ] ) ) return 0;
	
	glutMainLoop( );
}
