
// fx - A compositing library representation.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifdef HAVE_CONFIG_H
#include <openlibraries_global_config.hpp>
#endif

#include <iostream>

#include <openpluginlib/pl/openpluginlib.hpp>

#include <openeffectslib/fx/openeffectslib_plugin.hpp>

namespace oel = olib::openeffectslib;
namespace opl = olib::openpluginlib;

namespace
{
#ifdef WIN32
	const opl::string plugin_path( "./plugins" );
#else
	const opl::string plugin_path( OPENEFFECTSLIB_PLUGINS );
#endif

	struct query_traits
	{
		opl::wstring libname( ) const
		{ return L"openeffectslib"; }
		
		opl::wstring to_match( ) const
		{ return L"source"; }

		opl::wstring type( ) const
		{ return L""; }
		
		int merit( ) const
		{ return 0; }
	};

	oel::fx::n_operator_ptr op;
}

static bool create_operator( )
{
	typedef opl::discovery<query_traits> oel_discovery;
	
	oel_discovery plugins( ( query_traits( ) ) );
	if( plugins.empty( ) ) return false;
	
	oel_discovery::const_iterator i = plugins.begin( );
	boost::shared_ptr<oel::fx::openeffectslib_plugin> plug = boost::shared_dynamic_cast<oel::fx::openeffectslib_plugin>( i->create_plugin( "" ) );
	if( !plug ) return 0;

	op = plug->create( L"still image source" );
	if( !op ) return false;
	
	return true;
}

int main( )
{
	typedef oel::fx::n_operator::param_const_iterator param_const_iterator;
	
	opl::init( plugin_path );
	
	if( !create_operator( ) ) return 0;
	
	param_const_iterator I = op->param_begin( );
	while( I != op->param_end( ) )
	{
		std::wcout << "field name: " << I->first << " parameter type: " << I->second << "\n";
		++I;
	}
}
