
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include <cassert>
#include <iostream>

#include <openpluginlib/pl/pcos/property_container.hpp>
#include <openpluginlib/pl/pcos/observer.hpp>
#include <openpluginlib/pl/string.hpp>

namespace opl = olib::openpluginlib;
namespace pcos = olib::openpluginlib::pcos;

class counting_observer : public pcos::observer
{
public:
    counting_observer()
        : count( 0 )
        {}

    virtual void updated( pcos::subject* )
    {
        ++count;
    }

    unsigned int count;
};

int main( )
{
    // NULL containment test
    {
        pcos::property_container pc;
        assert( pc.getProperty( pcos::key::fromString( "key1" ) ) == pcos::property::NULL_PROPERTY );
        assert( pc.getProperty( "key1" ) == pcos::property::NULL_PROPERTY );
    }

    // basic containment test
    {
        pcos::property_container pc;
        pcos::property p1( pcos::key::fromString( "key1" ) );
        pcos::property p2( pcos::key::fromString( "key2" ) );
        assert( p1 != p2 );

        pc.append( p1 );
        pc.append( p2 );

        assert( pc.getKeys().size() == 2 );
        assert( pc.getProperty( "key1" ) == p1 );
        assert( pc.getProperty( "key2" ) == p2 );

        pc.remove( p2 );

        assert( pc.getKeys().size() == 1 );
        assert( pc.getProperty( "key1" ) == p1 );
        assert( pc.getProperty( "key2" ) == pcos::property::NULL_PROPERTY );
    }

    // notification
    {
        boost::shared_ptr< counting_observer > obs( new counting_observer );
        
        pcos::property_container pc;
        pcos::property p1( pcos::key::fromString( "key1" ) );
        pcos::property p2( pcos::key::fromString( "key2" ) );

        pc.append( p1 );
        pc.append( p2 );

        pc.attach( obs );
        
        p1 = 3.141;
        assert( obs->count == 1 );

        p2 = opl::wstring( L"foobar" );
        assert( obs->count == 2 );
    }

    return 0;
}
